#include <SoftwareSerial.h> // Software Serial Bibliothek einbinden

/////////////////////////////////////////////////////////////// Pins deklarieren

const int BTRX=2; // Arduino Recieve <- BT Send
const int BTTX=3; // Arduino Send -> BT Receive
const int motor1a=5;
const int motor1b=4;
const int motor1speed=6; // Motor PWM Pin
const int lichtpin=7;
const int statusled=9;
const int taster=10;
//const int standby=12; //wurde hier dauerhaft auf 5V gelegt, da gleiche Spannungsversorgung für Arduino und Motor


/////////////////////////////////////////////////////////////// Variablen deklarieren

int richtung=0;
boolean leuchten=false;
int gang=1;
int geschwindigkeit[7] = {0,50,90,130,170,210,255}; //PWM Werte für die Gänge 0-255
int tasterstatus=0;
int letztergang=1;
int bereit=0;

SoftwareSerial SerialBT(BTRX, BTTX); //Softwareserial für BT konfigurieren

int msg=999; // Wert des Empfangenen BT Befehls

/////////////////////////////////////////////////////////////// Fahrfunktionen

// vorwärts
void vor()
{
if (richtung==0) //Wenn die Lok steht
  { 
  analogWrite(motor1speed,0); // von welcher geschwindigkeit aus beschleunigt wird
  digitalWrite(motor1a, HIGH);
  digitalWrite(motor1b, LOW);
  richtung=1; //Richtung auf vor setzen
  
  beschleunigenvor(); // vorwärts Beschleunigen
  }
if (richtung==-1)  //Wenn die Lok rückwärts fährt
  {
  
  bremsen();
  digitalWrite(motor1a, HIGH);
  digitalWrite(motor1b, LOW);
  richtung=1; //Richtung auf vor setzen
  
  beschleunigenvor(); // vorwärts Beschleunigen
  }
}

// rückwärts fahren
void zurueck()
{
if (richtung==0)
  { 
  analogWrite(motor1speed,0); // von welcher geschwindigkeit aus beschleunigt wird   
  digitalWrite(motor1a, LOW);
  digitalWrite(motor1b, HIGH);
  richtung=-1;
  
  beschleunigenzurueck();
  }
if (richtung==1)
  {
  
  bremsen();
  digitalWrite(motor1a, LOW);
  digitalWrite(motor1b, HIGH);    
  richtung=-1;
  
  beschleunigenzurueck();
  }
}

// stopp
void stopp()
{
if (richtung!=0)
  {
  
  bremsen();
  digitalWrite(motor1a, LOW);
  digitalWrite(motor1b, LOW);
  richtung=0;
  }
}

// Licht toggeln

void licht()
{
  if (leuchten == false)  // wenn Licht aus ...
    {
      digitalWrite(lichtpin, HIGH); // ...dann anschalten...
      leuchten = true; // merke: es leuchtet
      Serial.println("Licht an"); // Licht an an PC senden
      SerialBT.write(20); //Code für Licht an an das Pult senden
      delay(30);
    } 
else 
    {
      digitalWrite(lichtpin, LOW); // ...sonst ausschalten
      leuchten = false; //merke: es leuchtet nicht
      Serial.println("Licht aus"); // Licht aus an PC senden
      SerialBT.write(21); //Code für Licht aus an das Pult senden
      delay(30);
    }
}

// Statusled Blink
void blinken()
{
  digitalWrite(statusled, LOW);
  delay(70);
  digitalWrite(statusled, HIGH);
  delay(70);
}

// bremsen bis zum Stillstand
void bremsen()
{
  Serial.println("bremsen");
      for (int aktuellegeschwindigkeit=geschwindigkeit[gang]; aktuellegeschwindigkeit>0; aktuellegeschwindigkeit--) // solange Geschwindigkeit da ist, diese verringern
    {
      analogWrite(motor1speed, aktuellegeschwindigkeit); // Geschwindigkeit setzen
      delay(2); // beeinflusst die Bremszeit
    }  
}

// beschleunigen vorwärts aus dem Stand
void beschleunigenvor()
{
  Serial.println("beschleunigen vorwärts");
     for (int aktuellegeschwindigkeit=0; aktuellegeschwindigkeit<geschwindigkeit[gang]; aktuellegeschwindigkeit++) // solange beschleunigen, bis eingestellte Geschwindigkeit erreicht ist
    {
      analogWrite(motor1speed, aktuellegeschwindigkeit); // Geschwindigkeit setzen
      delay(7); // beeinflusst die Beschleunigungszeit
    }
    letztergang=gang; //neuer Gang ist jetzt der letzte geschaltene Gang
}

// beschleunigen rückwärts aus dem Stand
void beschleunigenzurueck()
{
  Serial.println("beschleunigen rückwärts");
     for (int aktuellegeschwindigkeit=0; aktuellegeschwindigkeit<geschwindigkeit[gang]; aktuellegeschwindigkeit++) // solange beschleunigen, bis eingestellte Geschwindigkeit erreicht ist
    {
      analogWrite(motor1speed, aktuellegeschwindigkeit); // Geschwindigkeit setzen
      delay(7); // beeinflusst die Beschleunigungszeit
    }
    letztergang=gang;  //neuer Gang ist jetzt der letzte geschaltene Gang
}

// hoch schalten
void hochschalten()
{
     for (int aktuellegeschwindigkeit=geschwindigkeit[letztergang]; aktuellegeschwindigkeit<geschwindigkeit[gang]; aktuellegeschwindigkeit++) // solange beschleunigen, bis eingestellte Geschwindigkeit erreicht ist
    {
      analogWrite(motor1speed, aktuellegeschwindigkeit); // Geschwindigkeit setzen
      delay(5); // beeinflusst die Beschleunigungszeit
    }
    letztergang=gang; //neuer Gang ist jetzt der letzte geschaltene Gang
}

// runter schalten
void runterschalten()
{
     for (int aktuellegeschwindigkeit=geschwindigkeit[letztergang]; aktuellegeschwindigkeit>geschwindigkeit[gang]; aktuellegeschwindigkeit--) // solange abbremsen, bis eingestellte Geschwindigkeit erreicht ist
    {
      analogWrite(motor1speed, aktuellegeschwindigkeit); // Geschwindigkeit setzen
      delay(5); // beeinflusst die Bremszeit
    }
    letztergang=gang;  //neuer Gang ist jetzt der letzte geschaltene Gang
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Setup //////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void setup() {
SerialBT.begin(9600); // Startet die Bluetoothdatenübertragung mit 9600 Baud an die Lok

Serial.begin(9600); // Startet die Datenübertragung mit 9600 Baud an den PC
Serial.println("Verbindung steht");

pinMode(taster, INPUT); // Taster für fahren
pinMode(motor1a,OUTPUT); // Motor 1a
pinMode(motor1b,OUTPUT); // Motor 1b
pinMode(motor1speed,OUTPUT); // Motor 1c - PWM Pin
//pinMode(standby,OUTPUT); // Schalter für Motortreiber
pinMode(lichtpin,OUTPUT); // Licht
pinMode(statusled,OUTPUT); // Status LED

// initial alles auschalten

//digitalWrite(standby,HIGH); // Motortreiber aktivieren
digitalWrite(motor1a,LOW); // Motor 1a aus
digitalWrite(motor1b,LOW); // Motor 1b aus
analogWrite(motor1speed,geschwindigkeit[gang]); // Motor 1 startup Geschwindigkeit
digitalWrite(lichtpin,LOW); // Licht aus
digitalWrite(statusled,HIGH); // StatusLED an

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// LOOP ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void loop() {
if (bereit==0)
{
  SerialBT.write(99); //Code für Bereit aus an das Pult senden
  blinken();
  delay(200);
}
  
// Taster der Lok abfragen

tasterstatus = digitalRead(taster);
if (tasterstatus == HIGH)
{
  if (richtung==0)
  {
  Serial.println("vorwärts"); // Ausgabe auf der Seriellen schnittstelle
  gang=6;
  vor();
  }
  else
  {
  Serial.println("stopp"); // Ausgabe auf der Seriellen schnittstelle
  stopp();
  }
}
if ((SerialBT.available() > 0) && (SerialBT.available() < 100)) // wenn sinnvolle BT Daten empfangen werden
{      
     msg = SerialBT.read(); // Nachricht lesen
     Serial.println(msg);

switch(msg) // Abhaengig vom Signal in den richtigen Modus springen
{
 
 case 1: // Vorwärts
      Serial.println("vorwärts"); // Ausgabe auf der Seriellen schnittstelle
      vor();
      blinken();
 break;
 
 case 2: // Halt
      Serial.println("stopp"); // Ausgabe auf der Seriellen schnittstelle
      stopp();
      blinken();
 break;

 case 3: // Rückwärts
      Serial.println("rückwärts"); // Ausgabe auf der Seriellen schnittstelle
      zurueck();
      blinken();
 break;

 case 4: // Licht schalten
      Serial.println("licht"); // Ausgabe auf der Seriellen schnittstelle
      licht();
 break;

 case 11: // 11 Gang 1
      gang=1;
      Serial.println("Gang 1"); // Ausgabe auf der Seriellen schnittstelle
 break;

 case 12: // 12 Gang 2
      gang=2;
      Serial.println("Gang 2"); // Ausgabe auf der Seriellen schnittstelle
 break;

 case 13: // 13 Gang 3
      gang=3;
      Serial.println("Gang 3"); // Ausgabe auf der Seriellen schnittstelle
 break;

 case 14: // 14 Gang 4
      gang=4;
      Serial.println("Gang 4"); // Ausgabe auf der Seriellen schnittstelle
 break;

 case 15: // 15 Gang 5
      gang=5;
      Serial.println("Gang 5"); // Ausgabe auf der Seriellen schnittstelle
 break;

 case 16: // 16 Gang 6
      gang=6;
      Serial.println("Gang 6"); // Ausgabe auf der Seriellen schnittstelle
 break;

  case 99: // Bereit Empfangen
      bereit=1;
      Serial.println("Bereit Empfangen"); // Ausgabe auf der Seriellen schnittstelle
 break;
 
}
}
if((gang!=letztergang) && (richtung!=0))
{ 
  if (gang>letztergang)
  {
    Serial.println("hoch schalten"); // Ausgabe auf der Seriellen schnittstelle
    hochschalten();
  }

  if (gang<letztergang)
  {
    Serial.println("runter schalten"); // Ausgabe auf der Seriellen schnittstelle
    runterschalten();
  }  
  
}
delay(100);
}
